@@ -12,18 +12,17 @@ use crate::world::World;
 use crate::world::WorldAction;
 use crate::{traits::Mutator, traits::Serializer, Feature, FuzzedInput};
 
+use fuzzcheck_common::arg::{Arguments, FuzzerCommand};
 use fuzzcheck_common::{FuzzerEvent, FuzzerStats};
 
-use fuzzcheck_common::arg::{Arguments, FuzzerCommand};
 use libc::{SIGABRT, SIGALRM, SIGBUS, SIGFPE, SIGINT, SIGSEGV, SIGTERM};
 
+use std::borrow::Borrow;
 use std::panic::{catch_unwind, RefUnwindSafe, UnwindSafe};
 use std::path::Path;
 use std::process::exit;
 use std::result::Result;
 
-use std::borrow::Borrow;
-
 enum FuzzerInputIndex<T: Clone, M: Mutator<T>> {
     None,
     Temporary(FuzzedInput<T, M>),
@@ -565,9 +564,9 @@ where
 
     match command {
         FuzzerCommand::Fuzz => fuzzer.main_loop()?,
-        FuzzerCommand::MinifyInput { input_file } => fuzzer.input_minifying_loop(&input_file)?,
+        FuzzerCommand::MinifyInput { input_file } => fuzzer.input_minifying_loop(input_file)?,
         FuzzerCommand::Read { input_file } => {
-            let value = fuzzer.state.world.read_input_file(&input_file)?;
+            let value = fuzzer.state.world.read_input_file(input_file)?;
             if let Some((cache, mutation_step)) = fuzzer.state.mutator.validate_value(&value) {
                 fuzzer.state.input_idx = FuzzerInputIndex::Temporary(FuzzedInput::new(value, cache, mutation_step));
                 let input = FuzzerState::<T, M, S>::get_input(&fuzzer.state.input_idx, &fuzzer.state.pool).unwrap();
